<?php
/**
 * Admin functionality for Elegant URL Carousel
 */

class EUC_Admin {
    /**
     * Initialize the admin class
     */
    public function __construct() {
        add_action('admin_menu', array($this, 'add_admin_menu'));
        add_action('admin_init', array($this, 'register_settings'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_assets'));
        add_action('wp_ajax_euc_save_url', array($this, 'ajax_save_url'));
        add_action('wp_ajax_euc_delete_url', array($this, 'ajax_delete_url'));
        add_action('wp_ajax_euc_reorder_urls', array($this, 'ajax_reorder_urls'));
        add_action('wp_ajax_euc_import_urls', array($this, 'ajax_import_urls'));
    }

    /**
     * Add admin menu items
     */
    public function add_admin_menu() {
        add_options_page(
            __('Elegant URL Carousel', 'elegant-url-carousel'),
            __('URL Carousel', 'elegant-url-carousel'),
            'manage_options',
            'elegant-url-carousel',
            array($this, 'render_admin_page')
        );
    }

    /**
     * Register plugin settings
     */
    public function register_settings() {
        register_setting('euc_settings', 'euc_settings');

        add_settings_section(
            'euc_general_settings',
            __('General Settings', 'elegant-url-carousel'),
            null,
            'elegant-url-carousel'
        );

        // Add settings fields
        $this->add_settings_fields();
    }

    /**
     * Add settings fields
     */
    private function add_settings_fields() {
        $settings = array(
            'carousel_speed' => array(
                'label' => __('Carousel Speed (ms)', 'elegant-url-carousel'),
                'type' => 'number',
                'default' => 3000
            ),
            'visible_items' => array(
                'label' => __('Visible Items', 'elegant-url-carousel'),
                'type' => 'number',
                'default' => 3
            ),
            'primary_color' => array(
                'label' => __('Primary Color', 'elegant-url-carousel'),
                'type' => 'color',
                'default' => '#007bff'
            ),
            'animation_effect' => array(
                'label' => __('Animation Effect', 'elegant-url-carousel'),
                'type' => 'select',
                'options' => array(
                    'slide' => __('Slide', 'elegant-url-carousel'),
                    'fade' => __('Fade', 'elegant-url-carousel'),
                    'zoom' => __('Zoom', 'elegant-url-carousel')
                ),
                'default' => 'slide'
            )
        );

        foreach ($settings as $key => $setting) {
            add_settings_field(
                'euc_' . $key,
                $setting['label'],
                array($this, 'render_setting_field'),
                'elegant-url-carousel',
                'euc_general_settings',
                array(
                    'id' => $key,
                    'type' => $setting['type'],
                    'options' => isset($setting['options']) ? $setting['options'] : null,
                    'default' => $setting['default']
                )
            );
        }
    }

    /**
     * Render setting field
     */
    public function render_setting_field($args) {
        $options = get_option('euc_settings');
        $value = isset($options[$args['id']]) ? $options[$args['id']] : $args['default'];
        
        switch ($args['type']) {
            case 'number':
                printf(
                    '<input type="number" id="euc_%s" name="euc_settings[%s]" value="%s" class="regular-text">',
                    esc_attr($args['id']),
                    esc_attr($args['id']),
                    esc_attr($value)
                );
                break;
            
            case 'color':
                printf(
                    '<input type="color" id="euc_%s" name="euc_settings[%s]" value="%s">',
                    esc_attr($args['id']),
                    esc_attr($args['id']),
                    esc_attr($value)
                );
                break;
            
            case 'select':
                echo '<select id="euc_' . esc_attr($args['id']) . '" name="euc_settings[' . esc_attr($args['id']) . ']">';
                foreach ($args['options'] as $option_value => $option_label) {
                    printf(
                        '<option value="%s" %s>%s</option>',
                        esc_attr($option_value),
                        selected($value, $option_value, false),
                        esc_html($option_label)
                    );
                }
                echo '</select>';
                break;
        }
    }

    /**
     * Enqueue admin assets
     */
    public function enqueue_admin_assets($hook) {
        if ('settings_page_elegant-url-carousel' !== $hook) {
            return;
        }

        wp_enqueue_style(
            'euc-admin-style',
            EUC_PLUGIN_URL . 'assets/css/admin.css',
            array(),
            EUC_VERSION
        );

        wp_enqueue_script(
            'euc-admin-script',
            EUC_PLUGIN_URL . 'assets/js/admin.js',
            array('jquery', 'jquery-ui-sortable'),
            EUC_VERSION,
            true
        );

        wp_localize_script('euc-admin-script', 'eucAdmin', array(
            'ajaxUrl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('euc_admin_nonce')
        ));
    }

    /**
     * Render admin page
     */
    public function render_admin_page() {
        if (!current_user_can('manage_options')) {
            return;
        }

        // Get all URLs
        global $wpdb;
        $urls = $wpdb->get_results(
            "SELECT * FROM {$wpdb->prefix}euc_urls ORDER BY order_index ASC"
        );

        include EUC_PLUGIN_DIR . 'templates/admin-page.php';
    }

    /**
     * Handle AJAX URL save
     */
    public function ajax_save_url() {
        check_ajax_referer('euc_admin_nonce', 'nonce');
        
        if (!current_user_can('manage_options')) {
            wp_send_json_error('Unauthorized');
        }

        $url = sanitize_url($_POST['url']);
        if (!filter_var($url, FILTER_VALIDATE_URL)) {
            wp_send_json_error('Invalid URL');
        }

        // Process URL using URL Handler class
        $url_handler = new EUC_URL_Handler();
        $result = $url_handler->process_url($url);

        if (is_wp_error($result)) {
            wp_send_json_error($result->get_error_message());
        }

        wp_send_json_success($result);
    }

    /**
     * Handle AJAX URL deletion
     */
    public function ajax_delete_url() {
        check_ajax_referer('euc_admin_nonce', 'nonce');
        
        if (!current_user_can('manage_options')) {
            wp_send_json_error('Unauthorized');
        }

        $id = intval($_POST['id']);
        
        global $wpdb;
        $result = $wpdb->delete(
            $wpdb->prefix . 'euc_urls',
            array('id' => $id),
            array('%d')
        );

        if ($result === false) {
            wp_send_json_error('Failed to delete URL');
        }

        wp_send_json_success();
    }

    /**
     * Handle AJAX URL reordering
     */
    public function ajax_reorder_urls() {
        check_ajax_referer('euc_admin_nonce', 'nonce');
        
        if (!current_user_can('manage_options')) {
            wp_send_json_error('Unauthorized');
        }

        $order = array_map('intval', $_POST['order']);
        
        global $wpdb;
        foreach ($order as $index => $id) {
            $wpdb->update(
                $wpdb->prefix . 'euc_urls',
                array('order_index' => $index),
                array('id' => $id),
                array('%d'),
                array('%d')
            );
        }

        wp_send_json_success();
    }

    /**
     * Handle AJAX URL import
     */
    public function ajax_import_urls() {
        check_ajax_referer('euc_admin_nonce', 'nonce');
        
        if (!current_user_can('manage_options')) {
            wp_send_json_error('Unauthorized');
        }

        $urls = array_map('trim', explode("\n", $_POST['urls']));
        $results = array();
        
        $url_handler = new EUC_URL_Handler();
        
        foreach ($urls as $url) {
            if (empty($url)) continue;
            
            $url = sanitize_url($url);
            if (!filter_var($url, FILTER_VALIDATE_URL)) {
                $results[] = array(
                    'url' => $url,
                    'status' => 'error',
                    'message' => 'Invalid URL'
                );
                continue;
            }

            $result = $url_handler->process_url($url);
            
            if (is_wp_error($result)) {
                $results[] = array(
                    'url' => $url,
                    'status' => 'error',
                    'message' => $result->get_error_message()
                );
            } else {
                $results[] = array(
                    'url' => $url,
                    'status' => 'success',
                    'data' => $result
                );
            }
        }

        wp_send_json_success($results);
    }
}
