<?php
/**
 * URL Handler for Elegant URL Carousel
 */

class EUC_URL_Handler {
    /**
     * Process a URL and fetch its metadata
     */
    public function process_url($url) {
        // Validate URL
        if (!filter_var($url, FILTER_VALIDATE_URL)) {
            return new WP_Error('invalid_url', __('Invalid URL provided', 'elegant-url-carousel'));
        }

        // Initialize wp_remote_get args
        $args = array(
            'timeout' => 15,
            'sslverify' => true,
            'user-agent' => 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'
        );

        // Fetch URL content
        $response = wp_remote_get($url, $args);

        if (is_wp_error($response)) {
            return new WP_Error('fetch_failed', $response->get_error_message());
        }

        if (wp_remote_retrieve_response_code($response) !== 200) {
            return new WP_Error('invalid_response', __('Failed to fetch URL content', 'elegant-url-carousel'));
        }

        $content = wp_remote_retrieve_body($response);

        // Parse content
        libxml_use_internal_errors(true);
        $doc = new DOMDocument();
        @$doc->loadHTML($content);
        libxml_clear_errors();

        // Extract metadata
        $metadata = $this->extract_metadata($doc, $url);
        
        // Download and store favicon
        $favicon_path = $this->handle_favicon($metadata['favicon_url'], $url);
        $metadata['favicon_path'] = $favicon_path;

        // Store in database
        $result = $this->store_url_data($url, $metadata);

        if (is_wp_error($result)) {
            return $result;
        }

        return $metadata;
    }

    /**
     * Extract metadata from HTML content
     */
    private function extract_metadata($doc, $url) {
        $metadata = array(
            'title' => '',
            'description' => '',
            'favicon_url' => ''
        );

        // Extract title
        $title_tags = $doc->getElementsByTagName('title');
        if ($title_tags->length > 0) {
            $metadata['title'] = $this->sanitize_text($title_tags->item(0)->nodeValue);
        }

        // Extract meta description
        $meta_tags = $doc->getElementsByTagName('meta');
        foreach ($meta_tags as $meta) {
            if ($meta->getAttribute('name') === 'description') {
                $metadata['description'] = $this->sanitize_text($meta->getAttribute('content'));
                break;
            }
        }

        // Find favicon
        $favicon_url = $this->find_favicon($doc, $url);
        $metadata['favicon_url'] = $favicon_url;

        // Truncate description if needed
        if (strlen($metadata['description']) > 150) {
            $metadata['description'] = substr($metadata['description'], 0, 147) . '...';
        }

        return $metadata;
    }

    /**
     * Find favicon URL from HTML content
     */
    private function find_favicon($doc, $base_url) {
        $favicon_url = '';
        
        // Check for link tags
        $links = $doc->getElementsByTagName('link');
        foreach ($links as $link) {
            $rel = strtolower($link->getAttribute('rel'));
            if (in_array($rel, array('icon', 'shortcut icon', 'apple-touch-icon'))) {
                $favicon_url = $link->getAttribute('href');
                break;
            }
        }

        // If no favicon found, try default location
        if (empty($favicon_url)) {
            $parsed_url = parse_url($base_url);
            $favicon_url = $parsed_url['scheme'] . '://' . $parsed_url['host'] . '/favicon.ico';
        }

        // Make favicon URL absolute if it's relative
        if (!filter_var($favicon_url, FILTER_VALIDATE_URL)) {
            $favicon_url = $this->make_absolute_url($favicon_url, $base_url);
        }

        return $favicon_url;
    }

    /**
     * Convert relative URL to absolute URL
     */
    private function make_absolute_url($relative_url, $base_url) {
        $relative_url = trim($relative_url);
        
        // Return if already absolute
        if (parse_url($relative_url, PHP_URL_SCHEME) !== null) {
            return $relative_url;
        }

        $parsed_base = parse_url($base_url);
        
        // Handle protocol-relative URLs
        if (strpos($relative_url, '//') === 0) {
            return $parsed_base['scheme'] . ':' . $relative_url;
        }

        // Handle root-relative URLs
        if (strpos($relative_url, '/') === 0) {
            return $parsed_base['scheme'] . '://' . $parsed_base['host'] . $relative_url;
        }

        // Handle relative URLs
        $base_path = isset($parsed_base['path']) ? dirname($parsed_base['path']) : '/';
        return $parsed_base['scheme'] . '://' . $parsed_base['host'] . $base_path . '/' . $relative_url;
    }

    /**
     * Download and store favicon
     */
    private function handle_favicon($favicon_url, $site_url) {
        if (empty($favicon_url)) {
            return '';
        }

        // Create upload directory if it doesn't exist
        $upload_dir = wp_upload_dir();
        $favicon_dir = $upload_dir['basedir'] . '/euc-favicons';
        
        if (!file_exists($favicon_dir)) {
            wp_mkdir_p($favicon_dir);
        }

        // Generate unique filename
        $filename = md5($site_url) . '.png';
        $favicon_path = $favicon_dir . '/' . $filename;
        $favicon_url_path = $upload_dir['baseurl'] . '/euc-favicons/' . $filename;

        // Download favicon
        $response = wp_remote_get($favicon_url, array(
            'timeout' => 10,
            'sslverify' => true
        ));

        if (is_wp_error($response) || wp_remote_retrieve_response_code($response) !== 200) {
            return '';
        }

        $image_content = wp_remote_retrieve_body($response);
        
        // Save favicon
        if (file_put_contents($favicon_path, $image_content)) {
            return $favicon_url_path;
        }

        return '';
    }

    /**
     * Store URL data in database
     */
    private function store_url_data($url, $metadata) {
        global $wpdb;

        $data = array(
            'url' => $url,
            'title' => $metadata['title'],
            'description' => $metadata['description'],
            'favicon_path' => $metadata['favicon_path'],
            'last_fetched' => current_time('mysql'),
            'order_index' => $this->get_next_order_index()
        );

        $result = $wpdb->insert(
            $wpdb->prefix . 'euc_urls',
            $data,
            array(
                '%s', // url
                '%s', // title
                '%s', // description
                '%s', // favicon_path
                '%s', // last_fetched
                '%d'  // order_index
            )
        );

        if ($result === false) {
            return new WP_Error('db_error', __('Failed to store URL data', 'elegant-url-carousel'));
        }

        return true;
    }

    /**
     * Get next order index for new URLs
     */
    private function get_next_order_index() {
        global $wpdb;
        
        $max_order = $wpdb->get_var(
            "SELECT MAX(order_index) FROM {$wpdb->prefix}euc_urls"
        );

        return (int)$max_order + 1;
    }

    /**
     * Sanitize text content
     */
    private function sanitize_text($text) {
        $text = strip_tags($text);
        $text = html_entity_decode($text, ENT_QUOTES | ENT_HTML5, 'UTF-8');
        $text = preg_replace('/\s+/', ' ', $text);
        return trim($text);
    }
}
